/*
 * Copyright (c) 2003 wwcube@yahoo.co.jp
 */
/*
 * Copyright (c) 1999-2000, Qute Corporation. All rights reserved.
 */

/*
 *  FENT
 */

#include <stdio.h>
#include <string.h>
#include <time.h>

#include "tracelog.h"
#include "fent.h"

#define FENT_BLOCK_SIZE		128
#define FENT_FBFF_ID		"#!ws"
#define FENT_FBFF_ID_LEN	4
#define FENT_FBFF_PATCH_LEN	60

#define FENT_FMODE_COUNT	8
#define FENT_FMODE_SYMS		"xwrmsild"

#define FENT_RESID 0x5246

typedef struct {
	unsigned short magic;
	unsigned short type;
	unsigned short size;
	short id;
} res_t;

int find_bin(char *buf, long buflen, fent_t *binfentp, char **binbufpp, long *binbuflenp)
{
	fent_t *fentp;
	char *binbufp;
	long binbuflen;
	char name[FENT_MAXFNAME + 1];
	char info[FENT_MAXFINFO + 1];

	/* check data length */
	if (buflen < (FENT_FBFF_ID_LEN + FENT_FBFF_PATCH_LEN + sizeof(fent_t))) {
		tracelog(TRACELOG_LV_ERROR, "data too short\n");
		return -1;
	}

	/* check fx header */
	if (strncmp(buf, FENT_FBFF_ID, FENT_FBFF_ID_LEN) != 0) {
		tracelog(TRACELOG_LV_ERROR, "it is not a fx data\n");
		return -1;
	}

	/* read fent header */
	fentp = (fent_t *)(buf + FENT_FBFF_ID_LEN + FENT_FBFF_PATCH_LEN);
	binbufp = buf + FENT_FBFF_ID_LEN + FENT_FBFF_PATCH_LEN + sizeof(fent_t);
	{
		int i;
		struct tm *tm_p;
		long binlen;
		long reslen;
		long offset;

		strncpy(name, fentp->name, FENT_MAXFNAME);
		name[FENT_MAXFNAME] = '\0';
		strncpy(info, fentp->info, FENT_MAXFINFO);
		info[FENT_MAXFINFO] = '\0';
		tracelog(TRACELOG_LV_INFO, "name: %s\n", name);
		tracelog(TRACELOG_LV_INFO, "info: %s\n", info);
		tracelog(TRACELOG_LV_INFO, "mode: ");
		for (i = 0; i < FENT_FMODE_COUNT; i++) {
			if (fentp->mode & (1 << i)) {
				tracelog(TRACELOG_LV_INFO, "%c", FENT_FMODE_SYMS[i]);
			}
		}
		tracelog(TRACELOG_LV_INFO, "\n");
		tracelog(TRACELOG_LV_INFO, "size: %lu bytes (%d blocks)\n", fentp->len + FENT_BLOCK_SIZE, fentp->count + 1);
		tm_p = gmtime(&(fentp->mtime));
		tracelog(TRACELOG_LV_INFO, "date: %02d/%02d/%02d  %02d:%02d:%02d\n",
			(fentp->mtime >> 25) & 0x7f,
			(fentp->mtime >> 21) & 0x0f,
			(fentp->mtime >> 16) & 0x1f,
			(fentp->mtime >> 11) & 0x1f,
			(fentp->mtime >>  5) & 0x3f,
			(fentp->mtime <<  1) & 0x3f
		);

		offset = 0;
		if (fentp->resource > fentp->len) {
			tracelog(TRACELOG_LV_WARNING, "Warning: resource too long. ignore it\n");
			fentp->resource = -1;
		}
		binlen = (fentp->resource == -1) ? fentp->len : fentp->resource;
		reslen = (fentp->resource == -1) ? 0 : fentp->len - fentp->resource;
		binbuflen = binlen;
		if (fentp->resource != -1) {
			tracelog(TRACELOG_LV_INFO, "\noffset  bytes  type #id\n" \
				     "------ ------  --------\n"
			);
			if (binlen != 0) {
				tracelog(TRACELOG_LV_INFO, "%06X %6d  <code>\n", offset, binlen);
			}
			offset += binlen;
			{
				res_t *headerp;
				char resname[16];
				long len;
				for(;;) {
					headerp = (res_t *)(buf + offset + FENT_BLOCK_SIZE);
					if(headerp->magic == FENT_RESID) {
						strncpy(resname, (const char*)headerp, 4);
						resname[4] = '\0';
						len = (long)((unsigned long)headerp->size << 4);
						tracelog(TRACELOG_LV_INFO, "%06X %6d  %-4s #%d\r\n", offset, len, resname, headerp->id);
					} else {
						len = fentp->len - offset;
						tracelog(TRACELOG_LV_INFO, "%06X %6d  (unknown)\r\n", offset, len);
					}
					offset += len;
					if (fentp->len <= offset) {
						break;
					}
				}
			}
		}
		if (binfentp != NULL) {
			*binfentp = *fentp;
		}
		if (binbufpp != NULL) {
			*binbufpp = binbufp;
		}
		if (binbuflenp != NULL) {
			*binbuflenp = binbuflen;
		}
	}

	return 0;
}

