/*
 * Copyright (c) 2003 wwcube@yahoo.co.jp
 */

/*
 * Registory access
 */

#include <windows.h>

#include "tracelog.h"
#include "registory.h"

#define TM_SUBKEY_NAME			"Software\\Qute\\TransMagic"
#define TM_COMMPORT_VALUE_NAME	"CommPort"
#define TM_BAUDRATE_VALUE_NAME	"BaudRate"

int get_transmagic_settings(int *commportp, long *baudratep)
{
	HKEY hKey;
	long rc;
	DWORD commport;
	DWORD baudrate;
	DWORD type;
	DWORD len;

	/* Open Subkey */
	rc = RegOpenKeyEx(HKEY_CURRENT_USER, TM_SUBKEY_NAME, 0, KEY_READ, &hKey);
	if (rc != ERROR_SUCCESS){
		tracelog(TRACELOG_LV_WARNING, "Warning: RegOpenKeyEx() failed (error=%d)\n", rc);
		return -1;
	}

	/* Query Value */
	type = REG_DWORD;
	len = sizeof(commport);
	rc = RegQueryValueEx(hKey, TM_COMMPORT_VALUE_NAME, NULL, &type, (LPBYTE)&commport, &len);
	if (rc != ERROR_SUCCESS){
		RegCloseKey(hKey);
		tracelog(TRACELOG_LV_WARNING, "Warning: RegQueryValueEx(\"%s\") failed (error=%d)\n", TM_COMMPORT_VALUE_NAME, rc);
		return -1;
	}

	/* Query Value */
	type = REG_DWORD;
	len = sizeof(baudrate);
	rc = RegQueryValueEx(hKey, TM_BAUDRATE_VALUE_NAME, NULL, &type, (LPBYTE)&baudrate, &len);
	if (rc != ERROR_SUCCESS){
		RegCloseKey(hKey);
		tracelog(TRACELOG_LV_WARNING, "Warning: RegQueryValueEx(\"%s\") failed (error=%d)\n", TM_BAUDRATE_VALUE_NAME, rc);
		return -1;
	}

	RegCloseKey(hKey);
 
	*commportp = commport + 1;
	*baudratep = (baudrate == 0) ? 38400L : 9600L;

	return 0;
}
